maint <- function( N=100, u=0.001, stoptime=10,
                   init.A=1, init.C=0, init.G=0, init.T=0 ) {

   eps <- 1e-8

   # check N
   if ( N <= 0 )
      stop( "N must be greater than 0" )

   # check u
   if ( u > 1 || u < 0 )
      stop( "u must be between 0 and 1" )

   # check stoptime
   if ( stoptime <= 0 )
      stop( "stoptime must be greater than 0" )

   # check init values
   if ( init.A < 0 || init.A > 1 )
      stop( "init.A must be between 0 and 1" )
   if ( init.C < 0 || init.C > 1 )
      stop( "init.C must be between 0 and 1" )
   if ( init.G < 0 || init.G > 1 )
      stop( "init.G must be between 0 and 1" )
   if ( init.T < 0 || init.T > 1 )
      stop( "init.T must be between 0 and 1" )
   if ( sum( init.A, init.C, init.G, init.T ) > 1 + eps ||
        sum( init.A, init.C, init.G, init.T ) < 1 - eps )
      stop( "init values must sum up to 1" )


   bases <- c( "A", "C", "G", "T" )
   nn <- 2*N
   h <- 0
   h.pred <- 1 - ( 1 / ( 1 + 4*N*u ) )
   mutation <- FALSE
   fixed <- FALSE
   fix.hlp <- FALSE
   last.fixed <- 0

   freq <- matrix( c( init.A, init.C, init.G, init.T ), nrow=1, ncol=4 )
   colnames( freq ) <- bases

   h <- 1 - sum( freq[1,]^2 )
   h.pred <- 1 - ( 1 / ( 1 + 4*N*u ) ) 

   for ( i in 1:stoptime ) {
      nn.tmp <- nn
      freq <- rbind( freq, rep( 0, 4 ) )

      if ( runif( 1 ) < u*(1/0.75)*nn ) {
         # mutation occurs
         mut.base <- sample( bases, 1 )
         freq[i+1, mut.base] <- 1
         nn.tmp <- nn.tmp - 1
         mutation <- c( mutation, TRUE )
      } else {
         mutation <- c( mutation, FALSE )
      }

      freq[i+1,] <- ( freq[i+1,] + rmultinom( 1, size=nn.tmp, prob=freq[i,] ) )/ nn

      if ( any( freq[i+1,] == 1 ) ) {
         fixed.allele <- which( freq[i+1,] == 1 )
         if ( fixed.allele != last.fixed ) {
            # new fixation
            fixed <- c( fixed, TRUE )
            last.fixed <- fixed.allele
         }
         else {
            fixed <- c( fixed, FALSE )
         }
      } else {
         fixed <- c( fixed, FALSE )
      }
      
      h <- c( h, 1 - sum( freq[i+1,]^2 ) )
   }

   ret <- list( "freq"=freq, "h"=h, "h.pred"=h.pred,
                "numOfMut"=cumsum( mutation ), "fixed"=cumsum( fixed ) - 1 )
   class( ret ) <- "maintenance"

   return( ret )
}


plot.maintenance <- function( x, what=c( "A", "C", "G", "T" ), col=vector(),
                              xlim=NULL, ylim=NULL,
                              xlab="time", ylab="freq", 
                              legend.pos="bottomright", ... )
{
   stoptime = nrow( x$freq ) - 1

   legend <- vector()
   legend.col <- vector()

   col["A"] = ifelse( is.na( col["A"] ), "red", col["A"] )
   col["C"] = ifelse( is.na( col["C"] ), "blue", col["C"] )
   col["G"] = ifelse( is.na( col["G"] ), "green", col["G"] )
   col["T"] = ifelse( is.na( col["T"] ), "yellow", col["T"] )
   
   col["h"] = ifelse( is.na( col["h"] ), "grey", col["h"] )
   col["h.pred"] = ifelse( is.na( col["h.pred"] ), "black", col["h.pred"] )
   
   col["numOfMut"] = ifelse( is.na( col["numOfMut"] ), "darkblue", col["numOfMut"] )
   col["fixed"] = ifelse( is.na( col["fixed"] ), "lightblue", col["fixed"] )
   
   if ( is.null( xlim ) ) {
      xlim <- c( 0, stoptime )
   }
   
   if ( is.null( ylim ) ) {
     mx<-1
     if ( "fixed" %in% what ) {
       mx<-max(mx, x$fixed[stoptime]) 
     }
     if ( "numOfMut" %in% what ) {
       mx<-max(mx, x$numOfMut[stoptime]) 
     }
     ylim <- c( 0, mx )
   } 

   plot( 0,0, xlim=xlim, ylim=ylim, type="n",
         xlab=xlab, ylab=ylab, ... )

   if ( "A" %in% what ) {
      lines( 0:stoptime, x$freq[,"A"], col=col["A"] )
      legend <- c( legend, "A" )
      legend.col <- c( legend.col, col["A"] )
   }

   if ( "C" %in% what ) {
      lines( 0:stoptime, x$freq[,"C"], col=col["C"] )
      legend <- c( legend, "C" )
      legend.col <- c( legend.col, col["C"] )
   }

   if ( "G" %in% what ) {
      lines( 0:stoptime, x$freq[,"G"], col=col["G"] )
      legend <- c( legend, "G" )
      legend.col <- c( legend.col, col["G"] )
   }


   if ( "T" %in% what ) {
      lines( 0:stoptime, x$freq[,"T"], col=col["T"] )
      legend <- c( legend, "T" )
      legend.col <- c( legend.col, col["T"] )
   }

   if ( "h" %in% what ) {
      lines( 0:stoptime, x$h, col=col["h"] )
      legend <- c( legend, "h" )
      legend.col <- c( legend.col, col["h"] )
   }
   
   if ( "h.pred" %in% what ) {
      abline( h=x$h.pred, col=col["h.pred"] )
      legend <- c( legend, "h.pred" )
      legend.col <- c( legend.col, col["h.pred"] )
   }

   if ( "numOfMut" %in% what ) {
      lines( 0:stoptime, x$numOfMut, col=col["numOfMut"] )
      legend <- c( legend, "number of mutations" )
      legend.col <- c( legend.col, col["numOfMut"] )
   }

   if ( "fixed" %in% what ) {
      lines( 0:stoptime, x$fixed, col=col["fixed"] )
      legend <- c( legend, "number of fixated mutations" )
      legend.col <- c( legend.col, col["fixed"] )
   }
   
   legend( legend.pos, legend=legend,
           col=legend.col, lty=1, bg="white" )
}
